extern crate nihav_core;

use nihav_core::frame::*;
use std::io::prelude::*;
use std::fs::File;

pub fn write_pgmyuv(pfx: &str, strno: usize, num: u64, frm: NAFrameRef) {
    if let NABufferType::None = frm.get_buffer() { return; }
    let name = format!("{}out{:02}_{:08}.pgm", pfx, strno, num);
    let mut ofile = File::create(name).unwrap();
    let buf = frm.get_buffer().get_vbuf().unwrap();
    let (w, h) = buf.get_dimensions(0);
    let (w2, h2) = buf.get_dimensions(1);
    let has_alpha = buf.get_info().get_format().has_alpha();
    let tot_h;
    if has_alpha {
        tot_h = h * 2 + h2;
    } else {
        tot_h = h + h2;
    }
    let hdr = format!("P5\n{} {}\n255\n", w, tot_h);
    ofile.write_all(hdr.as_bytes()).unwrap();
    let dta = buf.get_data();
    let ls = buf.get_stride(0);
    let mut idx = 0;
    let mut idx2 = w;
    let mut pad: Vec<u8> = Vec::with_capacity((w - w2 * 2) / 2);
    pad.resize((w - w2 * 2) / 2, 0xFF);
    for _ in 0..h {
        let line = &dta[idx..idx2];
        ofile.write_all(line).unwrap();
        idx  += ls;
        idx2 += ls;
    }
    let mut base1 = buf.get_offset(1);
    let stride1 = buf.get_stride(1);
    let mut base2 = buf.get_offset(2);
    let stride2 = buf.get_stride(2);
    for _ in 0..h2 {
        let bend1 = base1 + w2;
        let line = &dta[base1..bend1];
        ofile.write_all(line).unwrap();
        ofile.write_all(pad.as_slice()).unwrap();

        let bend2 = base2 + w2;
        let line = &dta[base2..bend2];
        ofile.write_all(line).unwrap();
        ofile.write_all(pad.as_slice()).unwrap();

        base1 += stride1;
        base2 += stride2;
    }
    if has_alpha {
        let ls = buf.get_stride(3);
        let mut idx = buf.get_offset(3);
        let mut idx2 = idx + w;
        for _ in 0..h {
            let line = &dta[idx..idx2];
            ofile.write_all(line).unwrap();
            idx  += ls;
            idx2 += ls;
        }
    }
}

pub fn write_palppm(pfx: &str, strno: usize, num: u64, frm: NAFrameRef) {
    let name = format!("{}out{:02}_{:08}.ppm", pfx, strno, num);
    let mut ofile = File::create(name).unwrap();
    let buf = frm.get_buffer().get_vbuf().unwrap();
    let (w, h) = buf.get_dimensions(0);
    let paloff = buf.get_offset(1);
    let hdr = format!("P6\n{} {}\n255\n", w, h);
    ofile.write_all(hdr.as_bytes()).unwrap();
    let dta = buf.get_data();
    let ls = buf.get_stride(0);
    let offs: [usize; 3] = [
            buf.get_info().get_format().get_chromaton(0).unwrap().get_offset() as usize,
            buf.get_info().get_format().get_chromaton(1).unwrap().get_offset() as usize,
            buf.get_info().get_format().get_chromaton(2).unwrap().get_offset() as usize
        ];
    let mut idx  = 0;
    let mut line: Vec<u8> = Vec::with_capacity(w * 3);
    line.resize(w * 3, 0);
    for _ in 0..h {
        let src = &dta[idx..(idx+w)];
        for x in 0..w {
            let pix = src[x] as usize;
            line[x * 3 + 0] = dta[paloff + pix * 3 + offs[0]];
            line[x * 3 + 1] = dta[paloff + pix * 3 + offs[1]];
            line[x * 3 + 2] = dta[paloff + pix * 3 + offs[2]];
        }
        ofile.write_all(line.as_slice()).unwrap();
        idx  += ls;
    }
}

pub fn write_ppm(pfx: &str, strno: usize, num: u64, frm: NAFrameRef) {
    let name = format!("{}out{:02}_{:08}.ppm", pfx, strno, num);
    let mut ofile = File::create(name).unwrap();
    if let NABufferType::VideoPacked(ref buf) = frm.get_buffer() {
        let (w, h) = buf.get_dimensions(0);
        let hdr = format!("P6\n{} {}\n255\n", w, h);
        ofile.write_all(hdr.as_bytes()).unwrap();
        let dta = buf.get_data();
        let stride = buf.get_stride(0);
        let offs: [usize; 3] = [
                buf.get_info().get_format().get_chromaton(0).unwrap().get_offset() as usize,
                buf.get_info().get_format().get_chromaton(1).unwrap().get_offset() as usize,
                buf.get_info().get_format().get_chromaton(2).unwrap().get_offset() as usize
            ];
        let step = buf.get_info().get_format().get_elem_size() as usize;
        let mut line: Vec<u8> = Vec::with_capacity(w * 3);
        line.resize(w * 3, 0);
        for src in dta.chunks(stride) {
            for x in 0..w {
                line[x * 3 + 0] = src[x * step + offs[0]];
                line[x * 3 + 1] = src[x * step + offs[1]];
                line[x * 3 + 2] = src[x * step + offs[2]];
            }
            ofile.write_all(line.as_slice()).unwrap();
        }
    } else if let NABufferType::Video16(ref buf) = frm.get_buffer() {
        let (w, h) = buf.get_dimensions(0);
        let hdr = format!("P6\n{} {}\n255\n", w, h);
        ofile.write_all(hdr.as_bytes()).unwrap();
        let dta = buf.get_data();
        let stride = buf.get_stride(0);
        let depths: [u8; 3] = [
                buf.get_info().get_format().get_chromaton(0).unwrap().get_depth(),
                buf.get_info().get_format().get_chromaton(1).unwrap().get_depth(),
                buf.get_info().get_format().get_chromaton(2).unwrap().get_depth()
            ];
        let masks: [u16; 3] = [
                (1 << depths[0]) - 1,
                (1 << depths[1]) - 1,
                (1 << depths[2]) - 1
            ];
        let shifts: [u8; 3] = [
                buf.get_info().get_format().get_chromaton(0).unwrap().get_shift(),
                buf.get_info().get_format().get_chromaton(1).unwrap().get_shift(),
                buf.get_info().get_format().get_chromaton(2).unwrap().get_shift()
            ];
        let mut line: Vec<u8> = Vec::with_capacity(w * 3);
        line.resize(w * 3, 0);
        for src in dta.chunks(stride) {
            for x in 0..w {
                let elem = src[x];
                let r = ((elem >> shifts[0]) & masks[0]) << (8 - depths[0]);
                let g = ((elem >> shifts[1]) & masks[1]) << (8 - depths[1]);
                let b = ((elem >> shifts[2]) & masks[2]) << (8 - depths[2]);
                line[x * 3 + 0] = r as u8;
                line[x * 3 + 1] = g as u8;
                line[x * 3 + 2] = b as u8;
            }
            ofile.write_all(line.as_slice()).unwrap();
        }
    } else {
panic!(" unhandled buf format");
    }
}
